####################################################################################
#                                                                                  #
# Copyright (c) 2006 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#                                                                                  #
#  This file is globally imported into Picalo.  See picalo/__init__.py.            # 
#  When you write "from picalo import *", these functions all get imported.        #
#                                                                                  #
####################################################################################

from picalo import check_valid_table
from Error import error
import types
import ZODB.DB
import persistent
from persistent.dict import PersistentDict
from persistent.list import PersistentList


####################################################################
###   Represents a row of data



class Record(persistent.Persistent):
  '''An individual record of a Table. Individual fields in a record can be accessed 
     via column number or name.  In the table above, this is done with the following:
       >>> rec1 = mytable3[0]
       >>> print rec1[1]
       'Bart'
       >>> print rec1['Name'] 
       'Bart'
  '''
  def __init__(self, table):
    '''Called by Table's append or insert method.  Do not create Records directly.
       table         => The Picalo table that owns this record
    '''
    assert table != None, 'The table cannot be None when creating a record.'
    self.__dict__['_table'] = table  # can't use regular syntax because we override __setitem__ below
    self.__dict__['_rowdata'] = PersistentList()


  def __repr__(self):
    '''Returns a quick view of this record.  Provide mostly for debugging purposes.'''
    return '<Record: [%s]>' % ', '.join([ str(self[i]) for i in range(len(self.__dict__['_table'].columns)) ])
    
    
  def __str__(self):
    '''Returns a quick view of this record.  Provide mostly for debugging purposes.'''
    return '<Record: [%s]>' % ', '.join([ str(self[i]) for i in range(len(self.__dict__['_table'].columns)) ])
    
    
  def __iter__(self):
    '''Returns an iterator to this record (using the table's column order)'''
    return RecordIterator(self)
    
    
  def __contains__(self, item):
    '''Returns whether the given item is in the record'''
    for value in self:  # go through my cell values
      try:
        if item in value:
          return True
      except TypeError:  # occurs when we can't do the "in" keyword in the given value
        if item == value:
          return True
    return False
    
    
  def __eq__(self, other):
    '''Returns whether this record is equal to a list/tuple/record of values.
       The other parameter can be a record of this table, a record of another table,
       or any list-like object.  Only the values are compared -- the column names
       don't have to match (but the number of columns must be the same).
    '''
    if other != None and len(self) == len(other):
      for i in range(len(self)):
        if self[i] != other[i]:
          return False
      return True
    return False
    

  def __ne__(self, other):
    '''Returns whether this record is not equal to a list/tuple/record of values.
       The other parameter can be a record of this table, a record of another table,
       or any list-like object.  Only the values are compared -- the column names
       don't have to match (but the number of columns must be the same).
    '''
    return not self.__eq__(other)
  
  
    
  ##### CONTAINER METHODS ####
  
  
  def __len__(self):
    '''Returns the length of this record'''
    return self._table.column_count()
    
  
  def has_key(self, key):
    '''Returns whether the record has the given column name'''
    return self._table.columns_map_names.has_key(key)
  
  
  def __getattribute__(self, col):
    '''Retrieves the value of a field in the record.  This method allows
       record.colname type of access to values.  
       
       @param col: The column name or index
       @type  col: str
       @return:    The value in the given field
       @rtype:     returns
    '''
    # I've overridden __getattribute__ because persistent.Persistent.__getattribute__
    # incorrectly calls str(self) if the attribute is not found in the __dict__ of the
    # object.  In other words, __getattr__ never gets called as it should.
    if object.__getattribute__(self, '__dict__').has_key('_table') and object.__getattribute__(self, '_table').columns_map_names.has_key(col):
      return object.__getattribute__(self, '__get_value__')(col, [])
    try:
      return persistent.Persistent.__getattribute__(self, col)
    except AttributeError:
      raise AttributeError('This table has no column named "%s"' % col)
    
  
  def __getitem__(self, col):
    '''Retrieves the value of a field in the record.  This method allows
       record['colname'] type of access to values.
       
       @param col: The column name or index
       @type  col: str
       @return:    The value in the given field
       @rtype:     returns
    '''
    return self.__get_value__(col, [])
    
    
  def __get_value__(self, col, expression_backtrack):
    '''Internal method to retrieve the value of a cell.
       col                  => The column name or index
       expression_backtrack => A list of expression ids that have been run so far, to catch circular formulas
    '''
    # IMPORTANT: This function is called as much as any function in
    # Picalo.  It must be coded in the most efficient way possible.
    # (this entire file would be a good candidate to rewrite in C)

    # if a slice, recursively call me to get the items of the slice
    if isinstance(col, types.SliceType):
      return [ self.__getitem__(col.name) for col in self._table.columns[col] ]
    
    else:  # a regular single request
      ###   START OF NEAR DUPLICATE CODE   ###
      # There is duplicate code in Table.deref_column and Record.__set_item__.
      # If you change something here, modify both of those locations as well.
      # The duplicate code prevents extra method calls and speeds things up considerably.

      # if a name, convert to the index
      if not isinstance(col, int):
        if not col in self._table.columns_map_names:
          raise AttributeError('This table has no column named "%s"' % col)
        col = self._table.columns_map_names[col]
  
      # the column was referenced as a number
      else:
        origcol = col
        # if a negative index, convert to a positive one
        if col < 0:
          col += self._table.num_columns

        # ensure that we have enough columns to fulfill the request
        if col >= self._table.num_columns or col < 0:
          raise IndexError('Column index [%s] is out of range.  The table only has %s columns, referenced with [0] to [%s] or [-1] to [-%s].' % (origcol, self._table.num_columns, self._table.num_columns - 1, self._table.num_columns))

        # deref to the actual column location in the table
        col = self._table.columns_map_virtual[col]
    
      ###   END OF NEAR DUPLICATE CODE   ###  
    
      # get the column definition
      coldef = self._table.columns[col]
      
      # is it a calculated column?
      if coldef.expression:
        return coldef.expression.evaluate([{'record': self}, self], expression_backtrack)
        
      # if not a calculated column, just return the value (or None if the value for this column hasn't ever been set)
      # we don't actually store values in _data until something is set explicitly - None is implied and doesn't need to be stored
      if col < len(self.__dict__['_rowdata']):
        return self.__dict__['_rowdata'][col]
      else:
        return None
      
      
  def __setattr__(self, col, value):
    '''Sets the value of a field in the record.
      
       @param col:   The column name or index
       @type  col:   str/int
       @param value: The value to save in the field
       @type  value: value
    '''
    # first check to see if we have a column name
    if isinstance(col, (int, long)) or self._table.columns_map_names.has_key(col):
      self.__setitem__(col, value)
    # if not, then set as an attribute of the object
    persistent.Persistent.__setattr__(self, col, value)
    
  
  def __setitem__(self, col, value):
    '''Sets the value of a field in the record.
      
       @param col:   The column name or index
       @type  col:   str/int
       @param value: The value to save in the field
       @type  value: value
    '''
    assert not self._table.is_readonly(), 'This table is set as read-only and cannot be modified.'
    
    # if a slice, repeatedly call this method to set each member
    if isinstance(col, types.SliceType):
      for i in range(self._table.deref_column(col.start), self._table.deref_column(col.stop)):
        valueidx = i - self._table.deref_column(col.start)
        if valueidx >= 0 and valueidx < len(value):
          self.__setitem__(i, value[valueidx])
      return
    
    ###   START OF NEAR DUPLICATE CODE   ###
    # There is duplicate code in Record.__get_item__ and Table.deref_column.
    # If you change something here, modify both of those locations as well.
    # This method is called so much that the duplicate code speeds things up by several orders.

    # if a name, convert to the index
    if not isinstance(col, int):
      if not col in self._table.columns_map_names:
        raise IndexError('This table has no column named %s' % col)
      col = self._table.columns_map_names[col]
  
    # the column was referenced as a number
    else:
      origcol = col
      # if a negative index, convert to a positive one
      if col < 0:
        col += self._table.num_columns

      # ensure that we have enough columns to fulfill the request
      if col >= self._table.num_columns or col < 0:
        raise IndexError('Column index [%s] is out of range.  The table only has %s columns, referenced with [0] to [%s] or [-1] to [-%s].' % (origcol, self._table.num_columns, self._table.num_columns - 1, self._table.num_columns))

      # deref to the actual column location in the table
      col = self._table.columns_map_virtual[col]
    
    ###   END OF NEAR DUPLICATE CODE   ###  
    
    # get the column definition
    coldef = self._table.columns[col]
    
    # coerce the value into the right type for this column
    if value != None and not isinstance(value, coldef.column_type):
      try:
        value = coldef.parse_value(value)
      except Exception, e:
        value = error(e)
  
    # invalidate the indices now that we've changed data in this table
    self._table._invalidate_indexes()
  
    # grow the rowdata list to at least this value (we only do this when data are actually assigned to the row)
    while len(self.__dict__['_rowdata']) <= col:
      self.__dict__['_rowdata'].append(None)
    
    # set the value 
    self.__dict__['_rowdata'][col] = value 
    
    # notify listeners that we've had changes
    self._table._notify_listeners()
    
  
  def __setslice__(self, i, j, values):
    '''Sets the value of several items in this record.  Use slice notation:
       table[rec][0:2] = [ 1, 2 ]   # sets first two items
       
       @param i:  The starting index (inclusive)
       @type  i:  int
       @param j:  The ending index (exclusive)
       @type  j:  int
       @param values: A sequence of values to set
       @type  values: list or tuple
    '''
    # note: although setslice is deprecated, it is still in our superclass, so we have to override it
    self.__setitem__(slice(i,j), values)
   
   
###############################
###   Iterator for a record
   
def RecordIterator(record):     
  '''Returns a generator object to iterate over the columns of a record'''
  index = 0
  numcols = len(record._table.columns)
  while index < numcols:
    yield record[index]
    index += 1  





#!/usr/bin/env python
# -*- coding: windows-1251 -*-

#  Copyright (C) 2005 Roman V. Kiseliov
#  All rights reserved.
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
# 
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
# 
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
# 
#  3. All advertising materials mentioning features or use of this
#     software must display the following acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
# 
#  4. Redistributions of any form whatsoever must retain the following
#     acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
# 
#  THIS SOFTWARE IS PROVIDED BY Roman V. Kiseliov ``AS IS'' AND ANY
#  EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL Roman V. Kiseliov OR
#  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
#  OF THE POSSIBILITY OF SUCH DAMAGE.


__rev_id__ = """$Id: Row.py,v 1.6 2005/08/11 08:53:48 rvk Exp $"""


import BIFFRecords
from Deco import *
from Worksheet import Worksheet
import Style
import Cell
import ExcelFormula
import datetime as dt


class Row(object):
    __slots__ = ["__init__", 
                 "__adjust_height",
                 "__adjust_bound_col_idx",
                 "__excel_date_dt",
                 "get_height_in_pixels",
                 "set_style",
                 "get_xf_index",
                 "get_cells_count",
                 "get_min_col",
                 "get_max_col",
                 "get_str_count",
                 "get_row_biff_data",
                 "get_cells_biff_data",
                 "get_index",
                 "write",
                 "write_blanks",
                 # private variables
                 "__idx",
                 "__parent",
                 "__parent_wb",
                 "__cells",
                 "__min_col_idx",
                 "__max_col_idx",
                 "__total_str",
                 "__xf_index",
                 "__has_default_format",
                 "__height_in_pixels",
                 # public variables
                 "height",
                 "has_default_height",
                 "level",
                 "collapse",
                 "hidden",
                 "space_above",
                 "space_below"]

    #################################################################
    ## Constructor
    #################################################################
    def __init__(self, index, parent_sheet):
        self.__idx = index
        self.__parent = parent_sheet
        self.__parent_wb = parent_sheet.get_parent()
        self.__cells = []
        self.__min_col_idx = 0
        self.__max_col_idx = 0
        self.__total_str = 0
        self.__xf_index = 0x0F
        self.__has_default_format = 0
        self.__height_in_pixels = 0x11
        
        self.height = 0x00FF
        self.has_default_height = 0x00
        self.level = 0
        self.collapse = 0
        self.hidden = 0
        self.space_above = 0
        self.space_below = 0


    def __adjust_height(self, style):
        twips = style.font.height
        points = float(twips)/20.0
        # Cell height in pixels can be calcuted by following approx. formula:
        # cell height in pixels = font height in points * 83/50 + 2/5
        # It works when screen resolution is 96 dpi 
        pix = int(round(points*83.0/50.0 + 2.0/5.0))
        if pix > self.__height_in_pixels:
            self.__height_in_pixels = pix


    def __adjust_bound_col_idx(self, *args):
        for arg in args:
            if arg < self.__min_col_idx:
                self.__min_col_idx = arg
            elif arg > self.__max_col_idx:
                self.__max_col_idx = arg

    def __excel_date_dt(self, date):
        if isinstance(date, dt.date) and (not isinstance(date, dt.datetime)):
            epoch = dt.date(1899, 12, 31)
        elif isinstance(date, dt.time):
            date = dt.datetime.combine(dt.datetime(1900, 1, 1), date)
            epoch = dt.datetime(1900, 1, 1, 0, 0, 0)
        else:
            epoch = dt.datetime(1899, 12, 31, 0, 0, 0)
        delta = date - epoch
        xldate = delta.days + float(delta.seconds) / (24*60*60)
        # Add a day for Excel's missing leap day in 1900
        if xldate > 59:
            xldate += 1
        return xldate

    def get_height_in_pixels(self):
        return self.__height_in_pixels


    @accepts(object, Style.XFStyle)
    def set_style(self, style):
        self.__adjust_height(style)
        self.__xf_index = self.__parent_wb.add_style(style)

            
    def get_xf_index(self):
        return self.__xf_index

    
    def get_cells_count(self):
        return len(self.__cells)

    
    def get_min_col(self):
        return self.__min_col_idx

        
    def get_max_col(self):
        return self.__min_col_idx

        
    def get_str_count(self):
        return self.__total_str


    def get_row_biff_data(self):
        height_options = (self.height & 0x07FFF) 
        height_options |= (self.has_default_height & 0x01) << 15

        options =  (self.level & 0x07) << 0
        options |= (self.collapse & 0x01) << 4
        options |= (self.hidden & 0x01) << 5
        options |= (0x00 & 0x01) << 6
        options |= (0x01 & 0x01) << 8
        if self.__xf_index != 0x0F:
            options |= (0x01 & 0x01) << 7
        else:
            options |= (0x00 & 0x01) << 7
        options |= (self.__xf_index & 0x0FFF) << 16 
        options |= (0x00 & self.space_above) << 28
        options |= (0x00 & self.space_below) << 29
        
        return BIFFRecords.RowRecord(self.__idx, self.__min_col_idx, self.__max_col_idx, height_options, options).get()                                              
                        

    def get_cells_biff_data(self):
        return ''.join([ cell.get_biff_data() for cell in self.__cells ])


    def get_index(self):
        return self.__idx


    @accepts(object, int, (str, unicode, int, float, dt.datetime, dt.time, dt.date, ExcelFormula.Formula), Style.XFStyle)
    def write(self, col, label, style):
        self.__adjust_height(style)
        self.__adjust_bound_col_idx(col)
        if isinstance(label, (str, unicode)):
            if len(label) > 0:
                self.__cells.extend([ Cell.StrCell(self, col, self.__parent_wb.add_style(style), self.__parent_wb.add_str(label)) ])
                self.__total_str += 1
            else:
                self.__cells.extend([ Cell.BlankCell(self, col, self.__parent_wb.add_style(style)) ])
        elif isinstance(label, (int, float)):
            self.__cells.extend([ Cell.NumberCell(self, col, self.__parent_wb.add_style(style), label) ])            
        elif isinstance(label, (dt.datetime, dt.time)):
            self.__cells.extend([ Cell.NumberCell(self, col, self.__parent_wb.add_style(style), self.__excel_date_dt(label)) ])
        else:
            self.__cells.extend([ Cell.FormulaCell(self, col, self.__parent_wb.add_style(style), label) ])

    @accepts(object, int, int, Style.XFStyle)                        
    def write_blanks(self, c1, c2, style):
        self.__adjust_height(style)
        self.__adjust_bound_col_idx(c1, c2)
        self.__cells.extend([ Cell.MulBlankCell(self, c1, c2, self.__parent_wb.add_style(style)) ])

        
        


#!/usr/bin/python

####################################################################################
#                                                                                  #
# Copyright (c) 2003 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#
# Version 0.02
# Nov 15, 2004: Added routines from Aaron Bartholemew
# Sep 08, 2004: Added the cusum function.
# Dec 08, 2003: Version 0.01.  Started this log.
#
####################################################################################

from __future__ import division

__doc__ = '''
The Trending module contains functions that highlight trends in data.  Since fraud
is most often found in changes over time, this module is useful in looking at trends
over time.
'''

__functions__ = (
  'cusum',  
  'highlow_slope',
  'average_slope',
  'regression',
  'handshake_slope',
)

from math import sqrt
import re, types
from picalo import Table, TableArray, check_valid_table, run_tablearray, ensure_unique_colname


def cusum(table, col):
  '''Calculates a cusum, a cumulative difference in the values of a list
     at each row in the table.
     The cusum calculation gives a sense of the overall direction of a
     curve.
  
     Example:
     >>> table = Table([('col000', int), ('col001', int)], ([5,6], [3,2], [4,6]))
     >>> cusum = Trending.cusum(table, 0)  # cusum the first column (5, 3, 4)
     >>> cusum.view()
     +--------------+
     | col000_cusum |
     +--------------+
     |            0 |
     |           -2 |
     |           -1 |
     +--------------+

     @param table:  The table to be cusumed.
     @type  table:  Table or TableArray
     @param col:    The column name or index to cusum.
     @type  col:    str
     @return:       A new table containing a single column for the cusum value.
     @rtype:        Table
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Calculating...', cusum, table, col)
  check_valid_table(table, col)
  cusumcol = ensure_unique_colname(table, table.column(col).name + '_cusum')
  results = Table([(cusumcol, table.column(col).column_type)])
  results.append([0])
  for i in range(1, len(table)):
    results.append([table[i][col] - table[i-1][col] + results[-1][0]])
  return results
  
 

def highlow_slope(table, ycol, xcol=None):
  '''Computes a slope based on the minimum Y and the X that goes with it and the maximum Y and the X that goes with it. 
     Returns the X that goes with the minimum Y, the minimum Y, the X that goes with the maximum Y, the maximum Y, and the slope.
     
     If xcol is None, it is generated starting as 0, 1, 2, 3, etc.
 
     Example:
      >>> table = Table([('col000', unicode), ('col001', int), ('col002', int)], [
                     ['Dan',10,8],
                     ['Sally',12,12],
                     ['Dan',11,15], 
                     ['Sally',12,14], 
                     ['Dan',11,16], 
                     ['Sally',15,15], 
                     ['Dan',16,15], 
                     ['Sally',13,14]])
      >>> results = Trending.highlow_slope(table, 2, 1)
      >>> results.view()
      +------+------+------+------+-------+
      | MinX | MinY | MaxX | MaxY | Slope |
      +------+------+------+------+-------+
      |   10 |    8 |   11 |   16 |   8.0 |
      +------+------+------+------+-------+

     @param table: The table to calculate the slope on
     @type  table: Table
     @param ycol:  The y column to use.  The maximum and minimum are taken from this column.
     @type  ycol:  str
     @param xcol:  The x column to use.  This is optional.
     @type  xcol:  str
     @return:      A table with the first record giving the x and y of the minimum y, the x and y of the maximum y, and the slope betwee the two points.
     @rtype:       Table
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Calculating...', highlow_slope, table, ycol, xcol)
  check_valid_table(table, ycol)
  if xcol != None:
    check_valid_table(table, xcol)
  YPoints = table.column(ycol)
  ytype = table.column(ycol).column_type
  if xcol != None:
    XPoints = table.column(xcol)
    xtype = table.column(xcol).column_type
  else:
    XPoints = range(len(table))
    xtype = int
  MaxX=0
  MaxY=0
  MinX=0
  MinY=0
  count=0
  for Point in XPoints:
    Y=YPoints[count]
    X=Point
    if(count==0):
      MaxX=X
      MinX=X
      MaxY=Y
      MinY=Y
    if(MaxY<Y): 
      MaxX=X
      MaxY=Y
    if(MinY>Y): 
      MinX=X
      MinY=Y
    count=count+1
  ret = Table([('MinX', xtype), ('MinY', ytype), ('MaxX', xtype), ('MaxY', ytype), ('Slope', float)])
  r = ret.append()
  r['MinX'] = MinX
  r['MinY'] = MinY
  r['MaxX'] = MaxX
  r['MaxY'] = MaxY
  r['Slope'] = slope(MinX, MinY, MaxX, MaxY)
  return ret


def average_slope(table, ycol, xcol=None):
  '''Computes the average of the slopes between the points given.
     If xcol is None, it is generated starting as 0, 1, 2, 3, etc.

     Example:
      >>> from picalo import *
      >>> table = Table([('col000', unicode), ('col001', int), ('col002', int)], [
                     ['Dan',10,8],
                     ['Sally',12,12],
                     ['Dan',11,15], 
                     ['Sally',12,14], 
                     ['Dan',11,16], 
                     ['Sally',15,15], 
                     ['Dan',16,15], 
                     ['Sally',13,14]])
      >>> results = Trending.average_slope(table, 2, 1)
      >>> results.view()
      +-----------------+
      |  Average Slope  |
      +-----------------+
      | -0.559523809524 |
      +-----------------+

     @param table:  The table to calculate the slope on
     @type  table:  Table
     @param ycol:   The y column to use.  The maximum and minimum are taken from this column.
     @type  ycol:   str
     @param xcol:   The x column to use.  This is optional.
     @type  xcol:   str
     @return:       The average slope between points
     @rtype:        Table
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Calculating...', average_slope, table, ycol, xcol)
  check_valid_table(table, ycol)
  if xcol != None:
    check_valid_table(table, xcol)
  YPoints = table.column(ycol)
  if xcol != None:
    XPoints = table.column(xcol)
  else:
    XPoints = range(len(table))
  length = len(XPoints)-1  #zero based
  counter=0
  Sum = 0
  Total=[]
  while(counter<length):
    Total.append(slope(XPoints[counter], YPoints[counter], XPoints[counter+1], YPoints[counter+1]))
    counter=counter+1
  for set in Total:
    Sum=Sum+set
  return Table([('Average_Slope', float)], [[Sum/length]])


def slope(X1, Y1, X2, Y2):
  '''
     Calculated the slope between the two points given.
  '''
  assert isinstance(X1, (types.IntType, types.LongType, types.FloatType)), 'Invalid number: ' + X1
  assert isinstance(X2, (types.IntType, types.LongType, types.FloatType)), 'Invalid number: ' + X2
  assert isinstance(Y1, (types.IntType, types.LongType, types.FloatType)), 'Invalid number: ' + Y1
  assert isinstance(Y2, (types.IntType, types.LongType, types.FloatType)), 'Invalid number: ' + Y2
  return ((Y2-Y1)/(X2-X1))


def regression(table, ycol, xcol=None):
  '''Computes the regressionline for the points given.
     Returns the slope, intercept, correlation, and r-squared value of the regression line for the Points
     If xcol is None, it is generated starting as 0, 1, 2, 3, etc.

     Example:
      >>> from picalo import *
      >>> table = Table([('col000', unicode), ('col001', int), ('col002', int)], [
                     ['Dan',10,8],
                     ['Sally',12,12],
                     ['Dan',11,15], 
                     ['Sally',12,14], 
                     ['Dan',11,16], 
                     ['Sally',15,15], 
                     ['Dan',16,15], 
                     ['Sally',13,14]])
      >>> results = Trending.regression(table, 1)
      >>> results.view()
      +----------------+---------------+-----------------+------------------+
      |     Slope      |   Intercept   |   Correlation   |     RSquared     |
      +----------------+---------------+-----------------+------------------+
      | 0.619047619048 | 10.3333333333 | 0.0428888771398 | 0.00183945578231 |
      +----------------+---------------+-----------------+------------------+

     @param table:  The table to calculate the simple regression on
     @type  table:  Table
     @param ycol:   The y column to use  
     @type  ycol:   str
     @param xcol:   The x column to use.  This is optional.
     @type  xcol:   str
     @return:       A table containing the slope, intercept, correlation, and rSquared
     @rtype:        Table
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Calculating...', regression, table, ycol, xcol)
  check_valid_table(table, ycol)
  if xcol != None:
    check_valid_table(table, xcol)
  YPoints = table.column(ycol)
  if xcol != None:
    XPoints = table.column(xcol)
  else:
    XPoints = range(len(table))
  Ysum=0
  Xsum=0
  XSq=0
  XY=0
  slope=0
  length=len(XPoints)
  counter=0
  Returns=[]
  for Point in XPoints:
    X=Point
    Y=YPoints[counter]
    Ysum=Ysum+Y
    Xsum=Xsum+X
    XSq=XSq + X*X
    XY=XY+X*Y
    counter=counter+1
  rise=(length*XY)-(Xsum*Ysum)
  run=(length*XSq)-(Xsum*Xsum)
  slope=float(rise)/float(run)
  intercept = (Ysum - (slope*Xsum))/length
  correlation = ((length*XY)-(Xsum*Ysum))/sqrt(((length*XSq)-Xsum*Xsum)*(length*(Ysum*Ysum)-(Ysum*Ysum)))
  rSq = correlation * correlation
  ret = Table([('Slope', float), ('Intercept',float), ('Correlation', float), ('RSquared', float)])
  r = ret.append()
  r['Slope'] = slope
  r['Intercept'] = intercept
  r['Correlation'] = correlation
  r['RSquared'] = rSq
  return ret


def handshake_slope(table, ycol, xcol=None):
  '''Computes the slope between every point given.
     If xcol is None, it is generated starting as 0, 1, 2, 3, etc.

     For example:  Assume 5 points. 
     The slopes from points 1 to 2, 1 to 3, 1 to 4, 1 to 5, 2 to 3, 
     2 to 4, 2 to 5, 3 to 4, 3 to 5, and 4 to 5 are calculated.  
     The sum of those slopes are divided by the total number of 
     points to get an idea of the general trend.
       
       
     @param table:  The table to calculate the handshake slope on.
     @type  table:  Table
     @param ycol:   The y column to use.
     @type  ycol:   str
     @param xcol:   The x column to use.  This is optional.
     @type  xcol:   str
     @return:       A picalo table containing one cell: the calculated slope.
     @rtype:        Table
  '''
  if isinstance(table, TableArray):
    return run_tablearray('Calculating...', handshake_slope, table, ycol, xcol)
  check_valid_table(table, ycol)
  if xcol != None:
    check_valid_table(table, xcol)
  YPoints = table.column(ycol)
  if xcol != None:
    XPoints = table.column(xcol)
  else:
    XPoints = range(len(table))
  total=0
  runs=0
  counter=0
  for Point in XPoints:
    XPoint=Point
    YPoint=YPoints[counter]
    counter=counter+1
    inner_count=0
    for Set in XPoints:
      XSet=Set
      YSet=YPoints[inner_count]
      inner_count=inner_count+1
      if XPoint < XSet:
        PointSlope = (YSet - YPoint) / (XSet - XPoint)
        total = total + PointSlope
        runs=runs+1
  AverageSlope=total/runs
  return Table([('Handshake_Slope', float)], [[AverageSlope]])





#!/usr/bin/env python
# -*- coding: windows-1251 -*-

#  Copyright (C) 2005 Roman V. Kiseliov
#  All rights reserved.
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
# 
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
# 
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
# 
#  3. All advertising materials mentioning features or use of this
#     software must display the following acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
# 
#  4. Redistributions of any form whatsoever must retain the following
#     acknowledgment:
#     "This product includes software developed by
#      Roman V. Kiseliov <roman@kiseliov.ru>."
# 
#  THIS SOFTWARE IS PROVIDED BY Roman V. Kiseliov ``AS IS'' AND ANY
#  EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL Roman V. Kiseliov OR
#  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
#  OF THE POSSIBILITY OF SUCH DAMAGE.


__rev_id__ = """$Id: Style.py,v 1.4 2005/07/20 07:24:11 rvk Exp $"""


import Formatting
from BIFFRecords import *


_default_num_format = 'general'
_default_font = Formatting.Font()
_default_alignment = Formatting.Alignment()
_default_borders = Formatting.Borders()
_default_pattern = Formatting.Pattern()
_default_protection = Formatting.Protection()

class XFStyle(object):
    
    def __init__(self):
        self.num_format_str  = _default_num_format
        self.font            = _default_font 
        self.alignment       = _default_alignment
        self.borders         = _default_borders
        self.pattern         = _default_pattern 
        self.protection      = _default_protection

class StyleCollection(object):
    _std_num_fmt_list = [
            'general',
            '0',
            '0.00',
            '#,##0',
            '#,##0.00',
            '"$"#,##0_);("$"#,##',
            '"$"#,##0_);[Red]("$"#,##',
            '"$"#,##0.00_);("$"#,##',
            '"$"#,##0.00_);[Red]("$"#,##',
            '0%',
            '0.00%',
            '0.00E+00',
            '# ?/?',
            '# ??/??',
            'M/D/YY',
            'D-MMM-YY',
            'D-MMM',
            'MMM-YY',
            'h:mm AM/PM',
            'h:mm:ss AM/PM',
            'h:mm',
            'h:mm:ss',
            'M/D/YY h:mm',
            '_(#,##0_);(#,##0)',
            '_(#,##0_);[Red](#,##0)',
            '_(#,##0.00_);(#,##0.00)',
            '_(#,##0.00_);[Red](#,##0.00)',
            '_("$"* #,##0_);_("$"* (#,##0);_("$"* "-"_);_(@_)',
            '_(* #,##0_);_(* (#,##0);_(* "-"_);_(@_)',
            '_("$"* #,##0.00_);_("$"* (#,##0.00);_("$"* "-"??_);_(@_)',
            '_(* #,##0.00_);_(* (#,##0.00);_(* "-"??_);_(@_)',
            'mm:ss',
            '[h]:mm:ss',
            'mm:ss.0',
            '##0.0E+0',
            '@'   
    ]

    def __init__(self):
        self._fonts = {}
        self._fonts[Formatting.Font()] = 0
        self._fonts[Formatting.Font()] = 1
        self._fonts[Formatting.Font()] = 2
        self._fonts[Formatting.Font()] = 3
        # The font with index 4 is omitted in all BIFF versions
        self._fonts[Formatting.Font()] = 5
        
        self._num_formats = {}
        for fmtidx, fmtstr in zip(range(0, 23), StyleCollection._std_num_fmt_list[0:23]):
            self._num_formats[fmtstr] = fmtidx 
        for fmtidx, fmtstr in zip(range(37, 50), StyleCollection._std_num_fmt_list[23:]):
            self._num_formats[fmtstr] = fmtidx 

        self._xf = {}
        self.default_style = XFStyle()
        self._default_xf = self._add_style(self.default_style)[0]
        
    def add(self, style): 
        if style == None:
            return 0x10
        return self._add_style(style)[1]
    
    def _add_style(self, style):
        num_format_str = style.num_format_str
        if num_format_str in self._num_formats:
            num_format_idx = self._num_formats[num_format_str]
        else:
            num_format_idx = 163 + len(self._num_formats) - len(StyleCollection._std_num_fmt_list)
            self._num_formats[num_format_str] = num_format_idx
            
        font = style.font
        if font in self._fonts:
            font_idx = self._fonts[font]
        else:
            font_idx = len(self._fonts) + 1
            self._fonts[font] = font_idx
            
        xf = (font_idx, num_format_idx, style.alignment, style.borders, style.pattern, style.protection)
        
        if xf in self._xf:
            xf_index = self._xf[xf]
        else:
            xf_index = 0x10 + len(self._xf)
            self._xf[xf] = xf_index
            
        return xf, xf_index
        
    def get_biff_data(self):
        result = ''
        result += self._all_fonts()
        result += self._all_num_formats()
        result += self._all_cell_styles()
        result += self._all_styles()
        return result 
            
    def _all_fonts(self):
        result = ''
        i = sorted([(v, k) for k, v in self._fonts.items()])
        for font_idx, font in i:
            result += font.get_biff_record().get()
        return result
    
    def _all_num_formats(self):
        result = ''
        i = sorted([(v, k) for k, v in self._num_formats.items() if v>=163])
        for fmtidx, fmtstr in i:
            result += NumberFormatRecord(fmtidx, fmtstr).get()
        return result
    
    def _all_cell_styles(self):        
        result = ''
        for i in range(0, 16):
            result += XFRecord(self._default_xf, 'style').get()
            
        i = sorted([(v, k) for k, v in self._xf.items()])                            
        for xf_idx, xf in i:
            result += XFRecord(xf).get()
        return result
        
    def _all_styles(self):
        return StyleRecord().get()
        
if __name__ == '__main__':
    sc = StyleCollection()
    f = file('styles.bin', 'wb')
    f.write(sc.get_biff_data())
    f.close()

            


#!/usr/bin/python

####################################################################################
#                                                                                  #
# Copyright (c) 2003 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#
# Version 0.30
# Mar 05, 2004: Initial coding
#
#
#
####################################################################################

__doc__ = '''
Encodes financial ratios used for analysis of financial statements.
While most of these ratios are easy to code, this module provides 
a home for their standard coding.

These functions are useful in expressions used throughout Picalo.
'''

from picalo import check_valid_table
import types

# a listing of the public functions in this module (used by Manual.py)
__functions__ = [
  'current_ratio',
  'quick_ratio',
  'net_working_capital',
  'return_on_assets',
  'return_on_equity',
  'return_on_common_equity',
  'profit_margin',
  'earnings_per_share',
  'asset_turnover',
  'inventory_turnover',
  'debt_to_equity',
  'price_earnings',
]

def current_ratio(current_assets, current_liabilities):
  '''Returns the current ratio'''
  return float(current_assets) / float(current_liabilities)
  
  
def quick_ratio(current_assets, inventory, current_liabilities):
  '''Returns the quick ratio'''
  return float(current_assets - inventory) / float(current_liabilities)
  
  
def net_working_capital(current_assets, current_liabilities, total_assets):
  '''Returns the net working capital ratio'''
  return float(current_assets - current_liabilities) / float(total_assets)
  
  
def return_on_assets(net_income, beg_total_assets, end_total_assets):
  '''Returns the return on assets'''
  return float(net_income) / (float(beg_total_assets + end_total_assets) / 2.0)
  
  
def return_on_equity(net_income, beg_stockholders_equity, end_stockholders_equity):
  '''Returns the return on equity'''
  return float(net_income) / (float(beg_stockholders_equity + end_stockholders_equity) / 2.0)


def return_on_common_equity(net_income, beg_common_stockholders_equity, end_common_stockholders_equity):
  '''Returns the return on common equity'''
  return float(net_income) / (float(beg_common_stockholders_equity + end_common_stockholders_equity) / 2.0)


def profit_margin(net_income, sales):
  '''Returns the profit margin'''
  return float(net_income) / float(sales)
  
  
def earnings_per_share(net_income, number_of_shares):
  '''Returns the earnings per share'''
  return float(net_income, number_of_shares)
  
  
def asset_turnover(sales, beg_total_assets, end_total_assets):
  '''Returns the asset turnover ratio'''
  return float(sales) / (float(beg_total_assets + end_total_assets) / 2.0)
  
  
def inventory_turnover(cost_of_goods_sold, beg_inventory, end_inventory):
  '''Returns the inventory turnover ratio'''
  return float(cost_of_goods_sold) / (float(beg_inventory + end_inventory) / 2.0)
 
 
def debt_to_equity(total_liabilities, total_stockholders_equity):
  '''Returns the debt to equity ratio'''
  return float(total_liabilities) / float(total_stockholders_equity) 
  
  
def price_earnings(share_market_price, net_income, number_of_shares):
  '''Returns the PE (price earnings) ratio'''
  return float(share_market_price) / earnings_per_share(net_income, number_of_shares)
  




#!/usr/bin/env python
####################################################################################
#
# INSTRUCTIONS:
#
# To install Picalo system-wide (available to all Python programs),
# simply run:
#
#     python setup.py install
#
# from this directory.
#
#
####################################################################################
#                                                                                  #
# Copyright (c) 2003 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################


#from distutils.core import setup
from setuptools import setup
import glob, sys, fnmatch, os, os.path, shutil

# returns a list of all the files in a directory tree
def walk_dir(dirname, mask=''):
  files = []
  ret = [ (dirname, files) ]
  for name in os.listdir(dirname):
    fullname = os.path.join(dirname, name)
    if os.path.isdir(fullname):
      ret.extend(walk_dir(fullname))
    elif not mask:
      files.append(fullname)
    elif fnmatch.fnmatch(name, mask):
      files.append(fullname)
  return ret

# this manifest enables the standard Windows-looking theme
manifest = ''' 
<assembly xmlns="urn:schemas-microsoft-com:asm.v1" 
manifestVersion="1.0"> 
  <assemblyIdentity 
    version="0.6.8.0" 
    processorArchitecture="x86" 
    name="MyCare Card Browser" 
    type="win32" 
  /> 
  <description>MyCare Card Browser Program</description> 
  <trustInfo xmlns="urn:schemas-microsoft-com:asm.v3"> 
    <security> 
      <requestedPrivileges> 
        <requestedExecutionLevel 
          level="asInvoker" 
          uiAccess="false" 
        /> 
      </requestedPrivileges> 
    </security> 
  </trustInfo> 
  <dependency> 
    <dependentAssembly> 
      <assemblyIdentity 
        type="win32" 
        name="Microsoft.VC90.CRT" 
        version="9.0.21022.8" 
        processorArchitecture="x86" 
        publicKeyToken="1fc8b3b9a1e18e3b" 
      /> 
    </dependentAssembly> 
  </dependency> 
  <dependency> 
    <dependentAssembly> 
      <assemblyIdentity 
        type="win32" 
        name="Microsoft.Windows.Common-Controls" 
        version="6.0.0.0" 
        processorArchitecture="x86" 
        publicKeyToken="6595b64144ccf1df" 
        language="*" 
      /> 
    </dependentAssembly> 
  </dependency> 
</assembly> 
''' 

# get this version of Picalo
VERSION = open(os.path.join(os.path.dirname(__file__), 'picalo/VERSION.txt')).read().strip()

# The basic options for Picalo
options = {
  'name':             'picalo',
  'version':          VERSION,
  'description':      'A data analysis/structure library with tables, type-aware columns, records, and cells.',
  'long_description': open(os.path.join(os.path.dirname(__file__), 'picalo/README.txt')).read(),
  'author':           'Conan C. Albrecht',
  'author_email':     'conan@warp.byu.edu',
  'url':              'http://www.picalo.org/',
  'keywords':         'data log analysis spreadsheet database',  
  'license':          open(os.path.join(os.path.dirname(__file__), 'picalo/LICENSE.txt')).read(),
  'packages':         [ 
                        'picalo', 
                        'picalo.lib', 
                        'picalo.lib.pyExcelerator',
                      ],
  'install_requires': [
                         'ZODB3',
                         'zc.blist',
                      ],
  'classifiers':      [
                        "Development Status :: 4 - Beta",
                        'Environment :: Console',
                        'Intended Audience :: Developers',
                        'Intended Audience :: Science/Research',
                        'License :: OSI Approved :: GNU General Public License (GPL)',
                        'Operating System :: OS Independent',
                        'Topic :: Database',
                        'Topic :: Internet :: Log Analysis',
                        'Topic :: Office/Business :: Financial :: Spreadsheet',
                        'Topic :: Scientific/Engineering',
                        'Topic :: Text Processing',
                      ],                 
  'data_files':       [ 
                      ],
}

# add additional options if we're packging the GUI with it
if len(sys.argv) >= 2 and sys.argv[1] in ( 'py2exe', 'py2app' ):
  options['packages'].extend(
                       [
                         'picalo.gui',
                         'picalo.gui.wizards',
                       ] 
  )
  options['scripts'] = [
                         'Picalo.pyw',
                       ]
  options['package_data'] = {  
                        'picalo': [ 
                           'picalo',
                           'picalo/resources', 
                           'picalo/tools',
                           'picalo/tools/Detectlets',
                           'picalo/tools/Detectlets/BidRigging',
                           'picalo/tools/Detectlets/PhantomVendors',
                           'picalo/tools/Detectlets/SplitPurchases',
                         ] 
                       }
  options['data_files'].extend([ 
                            ('picalo/gui/wizards', glob.glob('picalo/gui/wizards/*')),
                            ('picalo/resources', glob.glob('picalo/resources/*')),
                            ('picalo/tools', glob.glob('picalo/tools/*.py')),
                          ] + walk_dir('picalo/tools/Detectlets', '*.py')),
                      

                      
# windows specific
if len(sys.argv) >= 2 and sys.argv[1] == 'py2exe':
  try:
    import py2exe
  except ImportError:
    print 'Could not import py2exe.   Windows exe could not be built.'
    sys.exit(0)
  # windows-specific options
  options['windows'] = [
    {
      'script':'Picalo.pyw',
      'icon_resources': [
        ( 1, 'picalo/resources/appicon-win.ico' ),
      ],
      'other_resources': [
        ( 24, 1, manifest ),
      ],
    },
  ]
  options['options'] = {
    'py2exe': { 
      'packages': [ 
        'picalo', 
        'picalo.lib', 
        'picalo.lib.pyExcelerator',
        'picalo.gui',
        'picalo.gui.wizards',
      ],
      'includes': 'psycopg2, MySQLdb, pyodbc, pgdb, cx_Oracle, sqlite3, chardet, email.iterators', # email.iterators is only because of a python 2.6 and py2exe bug (somehow they don't get into the archive automatically like everything else)
     }
  }
  options['data_files'].append(('.',              glob.glob('*.dll')))  # required dlls for libraries to run correctly
  options['data_files'].append(('.',              glob.glob('*.manifest')))  # required manifest for libraries to run correctly

# mac specific
if len(sys.argv) >= 2 and sys.argv[1] == 'py2app':
  try:
    import py2app
  except ImportError:
    print 'Could not import py2app.   Mac bundle could not be built.'
    sys.exit(0)
  # mac-specific options
  options['app'] = ['Picalo.pyw']
  options['options'] = {
    'py2app': {
      'argv_emulation': True,
      'iconfile': 'picalo/resources/appicon-mac.icns',
      'packages': [ 
        'picalo', 
        'picalo.lib', 
        'picalo.lib.pyExcelerator',
        'picalo.gui',
        'picalo.gui.wizards',
      ],
      'includes': 'psycopg2, MySQLdb, pgdb, sqlite3, chardet,email.iterators', # email.iterators is only because of a python 2.6 and py2exe bug (somehow they don't get into the archive automatically like everything else)
    }
  }


# run the setup
setup(**options)      



#!/usr/bin/env python

# GUID.py
# Version 2.8
#
# Copyright (c) 2008 Conan C. Albrecht
#
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal 
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished 
# to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all 
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
# FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
# DEALINGS IN THE SOFTWARE.

__doc__ = '''
  A globally-unique identifier made up of time and ip and 8 digits for a counter: 
  each GUID is 40 characters wide.

  A globally unique identifier that combines ip, time, and a counter.  Since the 
  time is listed first, you can sort records by guid.  You can also extract the time 
  and ip if needed.  

  Since the counter has eight hex characters, you can create up to 
  0xffffffff (4294967295) GUIDs every millisecond.  If your processor
  is somehow fast enough to create more than that in a millisecond (looking
  toward the future, of course), the function will wait until the next
  millisecond to return.
     
  GUIDs make wonderful database keys.  They require no access to the 
  database (to get the max index number), they are extremely unique, and they sort 
  automatically by time.   GUIDs prevent key clashes when merging
  two databases together, combining data, or generating keys in distributed
  systems.
  
  GUIDs are always returned as strings.  I did not make this a "real" class because
  it just complicates things.  Since GUIDs are generally stored in databases, sockets,
  and elsewhere as strings, I prefer to just keep them always as strings.  In other
  words, I don't see much benefit in having GUID objects over regular string objects.
  
  There is an Internet Draft for UUIDs, but this module does not implement it.
  If the draft catches on, perhaps I'll conform the module to it.
'''

# Changelog
# Sometime, 1997     Created the Java version of GUID
#                    Went through many versions in Java
# Sometime, 2002     Created the Python version of GUID, mirroring the Java version
# November 24, 2003  Changed Python version to be more pythonic, took out object and made just a module
# December 2, 2003   Fixed duplicating GUIDs.  Sometimes they duplicate if multiples are created
#                    in the same millisecond (it checks the last 100 GUIDs now and has a larger random part)
# December 9, 2003   Fixed MAX_RANDOM, which was going over sys.maxint
# June 12, 2004      Allowed a custom IP address to be sent in rather than always using the 
#                    local IP address.  
# November 4, 2005   Changed the random part to a counter variable.  Now GUIDs are totally 
#                    unique and more efficient, as long as they are created by only
#                    on runtime on a given machine.  The counter part is after the time
#                    part so it sorts correctly.
# November 8, 2005   The counter variable now starts at a random long now and cycles
#                    around.  This is in case two guids are created on the same
#                    machine at the same millisecond (by different processes).  Even though
#                    it is possible the GUID can be created, this makes it highly unlikely
#                    since the counter will likely be different.
# November 11, 2005  Fixed a bug in the new IP getting algorithm.  Also, use IPv6 range
#                    for IP when we make it up (when it's no accessible)
# November 21, 2005  Added better IP-finding code.  It finds IP address better now.
# January 5, 2006    Fixed a small bug caused in old versions of python (random module use)
# January 7, 2006    Reformatted the documentation to epydoc format
# February 28, 2007  Added the is_valid method
# March 6, 2007      Fixed the MAX_COUNTER so it would work in python 2.3 and lower.
# January 16, 2008   Added the decode() method.

import math
import socket
import random
import sys
import time
import threading
import types


#############################
###   global module variables

#Makes a hex IP from a decimal dot-separated ip (eg: 127.0.0.1)
make_hexip = lambda ip: ''.join(["%04x" % long(i) for i in ip.split('.')])
  
MAX_COUNTER = max(0xfffffffe, sys.maxint)
counter = 0L
firstcounter = MAX_COUNTER
lasttime = 0
hostip = ''
lock = threading.RLock()
try:  # only need to get the IP addresss once
  hostip = socket.getaddrinfo(socket.gethostname(),0)[-1][-1][0]
  hexip = make_hexip(hostip)
except: # if we don't have an ip, default to someting in the 10.x.x.x private range
  hostip = '10'
  rand = random.Random()
  for i in range(3):
    hostip += '.' + str(rand.randrange(1, 0xff))  
  hexip = make_hexip(hostip)

  
#################################
###   Public module functions

def generate(ip=None):
  '''Generates a new guid.  This is the primary method of the module.
     A guid is unique in space and time because it combines
     the machine IP with the current time in milliseconds.  It also includes
     a counter to provide uniqueness if two GUIDs are created in a single millisecond.
     
     Be careful about sending in
     a specified IP address because the ip makes it unique in space.  You could send in
     the same IP address that is created on another machine.  You should generally leave this
     parameter at its default value of None.
     
     @param      ip  An IP address to use in the GUID.  The default is the return from socket.getaddrinfo().
     @type       ip  str, such as '127.0.0.1' or a list, such as [ 127, 0, 0, 1 ]
     @returns:   A new GUID.
     @rtype:     str
  '''
  global counter, firstcounter, lasttime
  lock.acquire() # can't generate two guids at the same time
  try:
    parts = []

    # do we need to wait for the next millisecond (are we out of counters?)
    now = long(time.time() * 1000)
    while lasttime == now and counter == firstcounter: 
      time.sleep(.01)
      now = long(time.time() * 1000)

    # time part
    parts.append("%016x" % now)

    # counter part
    if lasttime != now:  # time to start counter over since we have a different millisecond
      firstcounter = long(random.uniform(1, MAX_COUNTER))  # start at random position
      counter = firstcounter
    counter += 1
    if counter > MAX_COUNTER:
      counter = 0
    lasttime = now
    parts.append("%08x" % (counter)) 

    # ip part
    parts.append(hexip)

    # put them all together
    return ''.join(parts)
  finally:
    lock.release()
    
    
def extract_time(guid):
  '''Extracts the time portion out of the guid and returns the 
     number of seconds since the epoch as a float.
  
     @param guid: The guid to extract the time from.
     @type  guid: str
     @returns:    The time this GUID was created as the number of seconds since the epoch (see the standard time module for more info).
     @rtype:      float
  '''
  return float(long(guid[0:16], 16)) / 1000.0


def extract_counter(guid):
  '''Extracts the counter from the guid.  Each GUID has a counter
     that ensures the GUID is unique, even if two GUIDs are created
     on the same comptuer at the same millisecond.  The counter is
     started at a random place in the range of an integer, so
     it's value is generally meaningless. :)
     
     @param guid: The guid to extract the counter from.
     @type  guid: str
     @returns:    The counter used when this GUID was created.
     @rtype:      int
  '''
  return int(guid[16:24], 16)


def extract_ip(guid):
  '''Extracts the ip portion out of the guid and returns it
     as a string like 10.10.10.10.  This generally provides 
     the machine IP where the GUID was created (unless the IP
     could not be determined at creation time).
     
     @param guid: The guid to extract the counter from.
     @type  guid: str
     @returns:    The IP address as something like "10.10.10.10".
     @rtype:      str
  '''
  # there's probably a more elegant way to do this
  thisip = []
  for index in range(24, 40, 4):
    thisip.append(str(int(guid[index: index + 4], 16)))
  return '.'.join(thisip)


def decode(guid):
  '''Extracts all parts of the guid and returns a tuple of three 
     items: time, counter, and ip.
     
     @param guid: The guid to extract the counter from.
     @type  guid: str
     @returns:    A tuple containing the (time, counter, and ip address)
     @rtype:      tuple
  '''
  return extract_time(guid), extract_counter(guid), extract_ip(guid)


def is_valid(guid):
  '''Returns whether the given guid seems to be a valid GUID or not.
     Since the guid is in string format, it can only check for appropriate
     length and valid characters in the GUID.
     
     @param guid: The guid to check.
     @type  guid: str
     @returns:    Whether it's a valid GUID or not
     @rtype:      bool
  '''
  if not isinstance(guid, types.StringTypes):
    return False
  if len(guid) != 40:
    return False
  for ch in guid.upper():
    if not ch in '0123456789ABCDEF':
      return False
  return True
    
  
  
### TESTING OF GUID CLASS ###
if __name__ == "__main__":
  num_to_create = int(raw_input('How many GUIDs shall I create? (default is 10)  ') or 10)
  guids = []
  for i in range(num_to_create):  # calculate very fast so people can see the counter in action
    guid = generate()
    guids.append(guid)
  for guid in guids:
    print "GUID:", guid
    guidtime = extract_time(guid)
    print "\tTime:   ", time.strftime('%a, %d %b %Y %H:%M:%S', time.localtime(guidtime)), '(millis: ' + str(round(guidtime - long(guidtime), 3)) + ')'
    print "\tIP:     ", extract_ip(guid)
    print "\tCounter:", extract_counter(guid)
  




# pyXLWriter: A library for generating Excel Spreadsheets
# Copyright (c) 2004 Evgeny Filatov <fufff@users.sourceforge.net>
# Copyright (c) 2002-2004 John McNamara (Perl Spreadsheet::WriteExcel)
#
# This library is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2.1 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
# General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#----------------------------------------------------------------------------
# This module was written/ported from PERL Spreadsheet::WriteExcel module
# The author of the PERL Spreadsheet::WriteExcel module is John McNamara
# <jmcnamara@cpan.org>
#----------------------------------------------------------------------------
# See the README.txt distributed with pyXLWriter for more details.

# Portions are (C) Roman V. Kiseliov, 2005

"""pyXLWriter.utilites

Utilities for work with reference to cells

"""
__rev_id__ = """$Id: Utils.py,v 1.1 2005/08/11 08:53:48 rvk Exp $"""

import re
from struct import pack
from ExcelMagic import MAX_ROW, MAX_COL


_re_cell_ex = re.compile(r"(\$?)([A-I]?[A-Z])(\$?)(\d+)")
_re_row_range = re.compile(r"\$?(\d+):\$?(\d+)")
_re_col_range = re.compile(r"\$?([A-I]?[A-Z]):\$?([A-I]?[A-Z])")
_re_cell_range = re.compile(r"\$?([A-I]?[A-Z]\$?\d+):\$?([A-I]?[A-Z]\$?\d+)")
_re_cell_ref = re.compile(r"\$?([A-I]?[A-Z]\$?\d+)")


def col_by_name(colname):
    """
    """
    col = 0
    pow = 1
    for i in xrange(len(colname)-1, -1, -1):
        ch = colname[i]
        col += (ord(ch) - ord('A') + 1) * pow
        pow *= 26
    return col - 1


def cell_to_rowcol(cell):
    """Convert an Excel cell reference string in A1 notation
    to numeric row/col notation.
  
    Returns: row, col, row_abs, col_abs
 
    """
    m = _re_cell_ex.match(cell)
    if not m:
        raise Exception("Error in cell format")
    col_abs, col, row_abs, row = m.groups()
    row_abs = bool(row_abs)
    col_abs = bool(col_abs)
    row = int(row) - 1
    col = col_by_name(col)
    return row, col, row_abs, col_abs


def cell_to_rowcol2(cell):
    """Convert an Excel cell reference string in A1 notation
    to numeric row/col notation.
  
    Returns: row, col
    
    """
    m = _re_cell_ex.match(cell)
    if not m:
        raise Exception("Error in cell format")
    col_abs, col, row_abs, row = m.groups()
    # Convert base26 column string to number
    # All your Base are belong to us.
    row = int(row) - 1
    col = col_by_name(col)
    return row, col
    
    
def rowcol_to_cell(row, col, row_abs=False, col_abs=False):
    """Convert numeric row/col notation to an Excel cell reference string in
    A1 notation.
 
    """
    d = col // 26
    m = col % 26
    chr1 = ""    # Most significant character in AA1
    if row_abs:
        row_abs = '$'
    else:
        row_abs = ''
    if col_abs:
        col_abs = '$'
    else:
        col_abs = ''
    if d > 0:
        chr1 = chr(ord('A') + d  - 1)
    chr2 = chr(ord('A') + m)
    # Zero index to 1-index
    return col_abs + chr1 + chr2 + row_abs + str(row + 1)


def cellrange_to_rowcol_pair(cellrange):
    """Convert cell range string in A1 notation to numeric row/col 
    pair.

    Returns: row1, col1, row2, col2
    
    """
    cellrange = cellrange.upper()
    # Convert a row range: '1:3'
    res = _re_row_range.match(cellrange)
    if res:
        row1 = int(res.group(1)) - 1
        col1 = 0
        row2 = int(res.group(2)) - 1
        col2 = -1
        return row1, col1, row2, col2
    # Convert a column range: 'A:A' or 'B:G'.
    # A range such as A:A is equivalent to A1:A16384, so add rows as required
    res = _re_col_range.match(cellrange)
    if res:
        col1 = col_by_name(res.group(1))
        row1 = 0
        col2 = col_by_name(res.group(2))
        row2 = -1
        return row1, col1, row2, col2
    # Convert a cell range: 'A1:B7'
    res = _re_cell_range.match(cellrange)
    if res:
        row1, col1 = cell_to_rowcol2(res.group(1))
        row2, col2 = cell_to_rowcol2(res.group(2))
        return row1, col1, row2, col2
    # Convert a cell reference: 'A1' or 'AD2000'
    res = _re_cell_ref.match(cellrange)
    if res:
        row1, col1 = cell_to_rowcol2(res.group(1))
        return row1, col1, row1, col1
    raise Exception("Unknown cell reference %s" % (cell))


def cell_to_packed_rowcol(cell):
    """ pack row and column into the required 4 byte format """
    row, col, row_abs, col_abs = cell_to_rowcol(cell)
    if col >= MAX_COL:
        raise Exception("Column %s greater than IV in formula" % cell)
    if row >= MAX_ROW: # this for BIFF8. for BIFF7 available 2^14
        raise Exception("Row %s greater than %d in formula" % (cell, MAX_ROW))
    col |= int(not row_abs) << 15
    col |= int(not col_abs) << 14
    return row, col



####################################################################################
#                                                                                  #
# Copyright (c) 2006 Dr. Conan C. Albrecht <conan_albrechtATbyuDOTedu>             #
#                                                                                  #
# This file is part of Picalo.                                                     #
#                                                                                  #
# Picalo is free software; you can redistribute it and/or modify                   #
# it under the terms of the GNU General Public License as published by             #
# the Free Software Foundation; either version 2 of the License, or                # 
# (at your option) any later version.                                              #
#                                                                                  #
# Picalo is distributed in the hope that it will be useful,                        #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                   #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    #
# GNU General Public License for more details.                                     #
#                                                                                  #
# You should have received a copy of the GNU General Public License                #
# along with Foobar; if not, write to the Free Software                            #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA        #
#                                                                                  #
####################################################################################
#                                                                                  #
#  This file is globally imported into Picalo.  See picalo/__init__.py.            # 
#  When you write "from picalo import *", these functions all get imported.        #
#                                                                                  #
####################################################################################

from picalo import check_valid_table, is_valid_variable, RESERVED_COLUMN_NAMES, VARIABLE_TYPES_RE, parse_value_to_type, format_value_from_type
from Expression import PicaloExpression
import types
from Calendar import Date, DateTime
from Error import error
from Number import number
import ZODB.DB
import persistent
from persistent.dict import PersistentDict
from persistent.list import PersistentList


####################################################
###   Column class

# Columns have both a virtual and actual location in the parent table's _data structure.
# When columns are added to a table, they are always added to the right side, even if they
# are inserted before other columns.  For example, if we have columns a, b, and c, inserting b1 
# between a and b results in actual positions a, b, c, b1.  That way we don't have to go through
# the entire record set and insert a None value to make room for the column.  Virtually, the 
# positioning will be a, b1, b, c so it accesses in the order the user expects.  When
# columns are deleted, we don't actually delete the data -- we just remove the column
# from the virtual list so it no longer shows.  It makes adding, removing, and deleting 
# columns super fast.  To actually delete the data, copy the table and erase the old one.

class Column(persistent.Persistent):
  '''A  single column of the table.  Columns are not copies of the values in the table,
     but rather pointers to the actual table data.  Therefore, changes to the values
     in the column are reflected in the underlying table.
     
     Columns must have unique names from other columns in the table.  Columns must also
     have either a column_type or an expression.
     
     Columns extend Python lists and mimick the list interface.  They can be used anywhere
     a regular Python list can.
  '''
  def __init__(self, table, index, name, column_type=None, expression=None, format=None):
    '''Creates a Column.  Do not call this method directly.  Rather, call mytable["colname"]'''
    check_valid_table(table)
    self.virtual_col_index = index  # the virtual location in the table or -1 if inactive
    self.table = table
    self.set_name(name)
    self.column_type = None
    self.expression = None
    self.format = None
    self.static_expression = None   # not used directly here, but stored so we can repopulate the TableProperties dialog
    self.set_type(column_type, format=format, expression=expression)
    
    
  def set_name(self, name):
    '''Sets the name of this column to the given name'''
    assert not self.table.is_readonly(), 'This table is set as read-only and cannot be modified.'
    assert isinstance(name, types.StringTypes), 'Invalid column name: ' + str(name)
    assert is_valid_variable(name), 'Invalid column name: %s.  Column names must start with a letter and then contain letters and numbers only.' % (str(name), )
    for col in self.table.columns:
      assert id(col) == id(self) or col.name != name, 'Two table columns cannot have the same name: ' + str(name)
    assert not RESERVED_COLUMN_NAMES.has_key(name), 'The name "' + str(name) + '" is a Picalo reserved word.  Please choose another name.'
    self.name = name
    self.table._calculate_columns_map()
    self.table.set_changed(True)
    self.table._notify_listeners()
    
    
  def __repr__(self):
    '''For debugging'''
    return '<Column instance: ' + self.name + '>'
    
    
  def __eq__(self, other):
    '''Compares this column with another column.  Everything must match except the table the two columns are bound to'''
    if isinstance(other, Column) and \
       self.name == other.name and \
       self.expression == other.expression and  \
       self.column_type == other.column_type:
         return True 
    return False


  def __ne__(self, other):
    '''Opposite of __eq__.  Checks whether two columns are not equals.'''
    return not self.__eq__(other)
    

  def set_type(self, column_type, format=None, expression=None):
    '''Sets the type of a column.   The type must be a valid <type> object,
       such as int, float, str, unicode, DateTime, etc.  All values in this
       column will be converted to this new type.
       
       @param column_type:  The new type of this column.
       @type  column_type:  type
       @param format:       A Picalo expression that evaluates to a string, using 'value' as the value of the field.
       @type  format:       str
       @param expression:   A Picalo expression that calculates this column. 
       @type  expression:   str
    '''
    assert not self.table.is_readonly(), 'This table is set as read-only and cannot be modified.'
    assert isinstance(column_type, (types.TypeType, types.ClassType)), '"%s" is an invalid column type for column "%s"' % (str(column_type), self.name)
    self.column_type = column_type
    self.static_expression = None
    if expression == None:   # going to a regular column type
      if self.expression != None:  # if previously an expression, record the values of the expression into the cells
        for i in range(len(self)):
          self.table[i][self.virtual_col_index] = self.expression.evaluate(({'record': self.table[i]}, self.table[i]))
        self.expression = None
      else: # going from a regular column type to a different regular column type
        for i in range(len(self)): # convert the current contents of the table
          self[i] = self[i]  # the __setitem__ in Record above will perform the expression
      
    else:  # going to a calculated column type
      if isinstance(expression, PicaloExpression):
        self.expression = PicaloExpression(expression.expression)
      else:
        self.expression = PicaloExpression(expression)
          
    self.set_format(format)
    # update indices
    self.table.set_changed(True)
    self.table._invalidate_indexes()
    self.table._notify_listeners(2)
    
    
  def get_type(self):
    '''Returns the type of this column.'''
    return self.column_type   


  def set_format(self, format=None):
    '''Sets the format of this column.  The format is used for printing and converting dates and for printing numbers.
       Regardless of the format, the cell keeps the full value internally and uses it in all Picalo functions.
       
       The format is made up of a mask consisting of special characters.  The characters are used
       to interpret values being converted or printed.
       
       The special characters used for Date and DateTime fields are as follows:
       
         - %a Locale's abbreviated weekday name.  
         - %A Locale's full weekday name.
         - %b Locale's abbreviated month name.  
         - %B Locale's full month name.  
         - %c Locale's appropriate date and time representation.  
         - %d Day of the month as a decimal number [01,31].  
         - %H Hour (24-hour clock) as a decimal number [00,23].  
         - %I Hour (12-hour clock) as a decimal number [01,12].  
         - %j Day of the year as a decimal number [001,366].  
         - %m Month as a decimal number [01,12].  
         - %M Minute as a decimal number [00,59].  
         - %p Locale's equivalent of either AM or PM. (1)
         - %S Second as a decimal number [00,61]. (2)
         - %U Week number of the year (Sunday as the first day of the week) as a decimal number [00,53]. All days in a new year preceding the first Sunday are considered to be in week 0. (3)
         - %w Weekday as a decimal number [0(Sunday),6].  
         - %W Week number of the year (Monday as the first day of the week) as a decimal number [00,53]. All days in a new year preceding the first Monday are considered to be in week 0. (3)
         - %x Locale's appropriate date representation.  
         - %X Locale's appropriate time representation.  
         - %y Year without century as a decimal number [00,99].  
         - %Y Year with century as a decimal number.  
         - %Z Time zone name (no characters if no time zone exists).  
         - %% A literal '%' character.
         
       The following are examples of date and time formats:
       
         - %a, %d %b %Y (Thu, 28 Jun 2001)
         - %B %d, %Y (June 28, 2001)
         - %Y-%m-%d (2015-12-30)
         - %Y/%m/%d (2015/12/30)
         - %a, %d %b %Y %H:%M:%S (Thu, 28 Jun 2001 14:17:15)
         - %B %d, %Y %H:%M:%S (June 28, 2001 14:17:15)
         - %Y-%m-%d %H:%M:%S (2015-12-30 14:17:15)
         - %Y/%m/%d %H:%M:%S (2015/12/30 14:17:15)
       
       The special characters used for all number fields (int, long, float, number, etc.) are as follows:
       
         - Prefix the format with any character(s) (like $) to add to the front of the number.
         - End the format with a zero (0) to round to the nearest whole number.
         - Use a period (.) to denote decimal portions of the number.
         - Use a pound (#) to denote a regular number.
         - Use a comma (,) to denote a thousands separator (use with # signs to place it every three numbers)
         - Use a percent (%) to denote the value should be displayed as a percent (multiplied by 100 to when displayed and divide by 100 when parsing input and the number ends with a %)
         - Use the letter E+ followed by zeros to denote scientific notation. 
         
       The following are examples of number formats:
       
         - 0 (round to the nearest whole number;  10.99 is displayed as 11; 12.3 is displayed as 12)
         - 0.00 (rounds to the nearest hundredth; 10.99 is formatted as 10.99; 12.309 is formatted as 12.31; 13 is formatted as 13.00)
         - $0.00 (rounds to the nearest hundredth and adds a dollar sign to the front of the number; you can also use any other character, such as the euro glyph)
         - #,### (formats thousands with a comma;  1234.56 is formatted as 1,234.56)
         - #,##0 (formats thousands with a comma and rounds to the nearest whole number; 1234.56 is formatted as 1,235)
         - #,##0.000 (formats thousands with a comma and rounds to the nearest thousandth; 1234.56 is formatted as 1,234.560)
         - 0% (rounds to the nearest whole number, multiplies by 100 (for display only), and adds a percent sign)
         - 0.00% (rounds to the nearest hundredth, multiplies by 100 (for display only), and adds a percent sign)
         - #E+000 (shows the number in scientific notation to the given number of decimal places)
         
       As of right now, the following formatting options are not available:
         - Numbers with a comma for the decimal separator and a period as the thousands separator.
         - Parentheses for negative numbers.
       
       To set the format on a field, set the format as follows:
         table.column('colname').set_format('#,##0')
       or
         table.set_format('colname', '#%')
       
       To remove the format from a field, set the format to None, as in:
         table.column('colname').set_format(None)
       or
         table.set_format('colname', None)
       
       @param format:  A format mask using the special characters described in the documentation.
       @type  format:  str
    '''
    # add the output function to the column
    self.format = format
    self.table.set_changed(True)
    self.table._notify_listeners(2)
    
    
  def get_format(self):
    '''Returns the format of this column.  If no format has been set, 
       None is returned.'''
    return self.format
    
    
  def format_value(self, value):
    '''Format the given value according to this column's set format.  This method is used when
       showing values in Picalo cells, when exporting, and in pretty printing.  The method
       always returns a unicode string.'''
    return format_value_from_type(value, self.column_type, self.format)
       
       
  def parse_value(self, value):
    '''Parses the given value according to this column's set format.  This method is used when
       importing data into the field.  If the given field is already the correct type,
       it is not parsed but is simply returned.
       
       Picalo uses the format extensively when parsing dates.  However, it always tries to parse
       numbers regardless of the format.  In other words, if a number being placed in a cell contains
       a percent sign, Picalo will divide by 100 when setting the value in the cell.  If a number
       contains scientific notation (ex: 1.23e+45), Picalo will expand the notation, even if a format
       has not been set on a field.
       
       This method is normally called by Record.__setitem__ (whenever a value is set in a table cell).
    '''
    return parse_value_to_type(value, self.column_type, self.format)
    
    

    
  ##### DISABLED LIST METHODS ####  
  def __delitem__(self, col):  raise 'Not allowed on Columns.  Delete at the table level.'
  def append(self, *a, **k):   raise 'Not allowed on Columns.  Append at the table level.'
  def extend(self, *a, **k):   raise 'Not allowed on Columns.  Extend at the table level.'
  def insert(self, *a, **k):   raise 'Not allowed on Columns.  Insert at the table level.'
  def pop(self, *a, **k):      raise 'Not allowed on Columns.  Pop at the table level.'
  def remove(self, *a, **k):   raise 'Not allowed on Columns.  Remove at the table level.'
  def reverse(self, *a, **k):  raise 'Not allowed on Columns.  Reverse not supported.'
  def sort(self, *a, **k):     raise 'Not allowed on Columns.  Sort at the table level.'

  ##### CONTAINER METHODS ####
  
  def __len__(self):
    '''Count the number of records in the column.
    
       @return:  The number of records.  This is the same value as the number of records in the underlying table.
       @rtype:   returns
    '''
    return len(self.table)
    
  def __getitem__(self, recindex):
    '''Retrieves the value of a field in the given record index.
       
       @param    recindex: the record index where the value will be retrieved.
       @type     int
       @return:  The value of the field.
       @rtype:   object
    '''
    assert isinstance(recindex, (types.IntType, types.LongType)), 'Invalid record index: ' + str(num_records)
    return self.table[recindex][self.virtual_col_index]
    
  def __setitem__(self, recindex, value):
    '''Sets a value in the column.  The value of this cell in the underlying table
       is changed.
    
       @param recindex: The record in dex where the value will be stored.
       @type  recindex: int
       @param value: The value to set in the table.
       @type  value: object
    '''
    assert isinstance(recindex, (types.IntType, types.LongType)), 'Invalid record index: ' + str(num_records)
    self.table[recindex][self.virtual_col_index] = value
    
  def __iter__(self):
    return ColumnIterator(self)
  
  
  def __contains__(self, item):
    '''Returns true if the item is in the column'''
    for value in self:  # go through my cell values
      try:
        if item in value:
          return True
      except TypeError:  # occurs when we can't do the "in" keyword in the given value
        if item == value:
          return True
    return False
    

  def _get_columnloader(self):
    '''Returns a column loader for this column (a representation of this column without ties to any table.
       This is normally only used internally in Picalo.
    '''
    column = _ColumnLoader()
    column.name = self.name
    column.column_type = self.column_type
    column.expression = self.expression != None and self.expression.expression or None
    column.format = self.format
    return column
  
  
# _ColumnLoader is used in:
#   1. _loadVersion1 
#   2. Simple.join
#   3. Project.save and Project.load
class _ColumnLoader:
  '''Little class that helps in loading columns'''
  def __init__(self, name=None, column_type=None, expression=None, format=None):
    self.name = name
    self.column_type = column_type
    self.expression = expression
    self.format = format
  


  
##################################################################
###   An iterator for columns
      
def ColumnIterator(column, respect_filter=True):     
  '''Returns a generator object to iterate over a table'''
  index = 0
  table = column.table
  colindex = table.deref_column(column.name)
  numrows = table.record_count(respect_filter)
  while index < numrows:
    yield table.record(index, respect_filter)[colindex]
    index += 1       
      


  